*///////////////////////////////////////////////////////////////
*// Author:       Michael Drozdov
*// Description:  Demonstrate: 
*//               change of the order of sorting 
*//               on click on header of a column in Grid
*// VFP version:  VFP 8.0 (or later) 
*// BaseClass:    Header
*// ParentClass:  Header
*// Time Stamp:   08/09/06 07:26:12 PM
*///////////////////////////////////////////////////////////////
*// 
#DEFINE N_SCROLL_UP	5
DEFINE CLASS MyHeader as Header
	Name = 'MyHeader'
	Alignment = 2                   &&<- Initial type alignment for header
	BackColor = RGB(192,192,192)    &&<- Initial header BackColor
	cPicDn = 'Graphics\OrderDn.bmp' &&<- Picture for ASCENDING
	cPicUp = 'Graphics\OrderUp.bmp' &&<- Picture for DESCENDING
	PROTECTED cAlias                &&<- Alias in Colunm.ControlSource        
	PROTECTED cField                &&<- Field in Colunm.ControlSource        
	PROTECTED cTag                  &&<- TagName for field in Colunm.ControlSource        
	
	PROCEDURE Init
		WITH this
			STORE '' TO ;
				.cAlias, .cField, .cTag
		ENDWITH	
	ENDPROC
	
	*/////////////////////////////////////////////////////////////////
	*// ClearAllPictures() - "removes pictures" at all headers in grid   
	*/////////////////////////////////////////////////////////////////
	PROCEDURE ClearAllPictures 
		IF TYPE('This.Parent.Parent') # 'O' OR ISNULL(This.Parent.Parent)
			RETURN .F.
		ENDIF	 

		LOCAL oGrd as Grid;
			,lnCntCol as Number, loCol as Column;
			,lnCntCon as Number, loCon as Object
		oGrd = This.Parent.Parent
		FOR lnCntCol = 1 TO oGrd.ColumnCount
			loCol = oGrd.Columns(lnCntCol)
			IF VARTYPE(loCol) = 'O'
				FOR lnCntCon = 1 TO loCol.ControlCount
				  	loCon = loCol.Controls(lnCntCon)
				  	IF VARTYPE(loCon) = 'O' ;
				  			AND 'Header' $ loCon.BaseClass
						IF !EMPTY(loCon.Picture)
							loCon.Picture = ''
						ENDIF
						EXIT
					ENDIF
				ENDFOR
			ENDIF
		ENDFOR
	ENDPROC

	*//////////////////////////////////////////////////////////
	*// GetAlias() - return the alias used as a source of data 
	*// for the instance of the class-header
	*//////////////////////////////////////////////////////////
	PROCEDURE GetAlias()
		RETURN this.cAlias
	ENDPROC

	*//////////////////////////////////////////////////////////
	*// GetCdxFile(tcAlias) - return the name of a cdx-file 
	*// from the name of a dbf-file at the alias of the table 
	*// tcAlias, passed as parameter
	*//////////////////////////////////////////////////////////
	PROCEDURE GetCdxFile
		LPARAMETERS tcAlias

		LOCAL lcCdxFile as String
		lcCdxFile = ''
		IF VARTYPE(tcAlias) # 'C' OR EMPTY(tcAlias)
			RETURN lcCdxFile
		ENDIF	 

		IF USED(tcAlias)   
			LOCAL lcDbfFile as String
			lcDbfFile = LOWER(DBF(tcAlias))
			lcCdxFile = FORCEEXT(lcDbfFile, 'cdx')
		ENDIF	

		RETURN lcCdxFile
	ENDPROC	

	*//////////////////////////////////////////////////////////
	*// GetCdxFile(tcAlias) - delete the cdx-file for an alias 
	*// passed as parameter (if such is available)
	*//////////////////////////////////////////////////////////
	PROCEDURE DeleteCdxFile
		LPARAMETERS tcAlias

		LOCAL lbRetVal as Boolean 
		IF VARTYPE(tcAlias) # 'C' OR EMPTY(tcAlias)
			RETURN lbRetVal
		ENDIF	 

		IF USED(tcAlias)   
			LOCAL lcCdxFile as String
			lcCdxFile = this.GetCdxFile(tcAlias)
			IF FILE(lcCdxFile)
				LOCAL lnOldSel as Number
				lnOldSel = SELECT() 
				SELECT (tcAlias)
				SET ORDER TO 0
				DELETE TAG ALL OF (lcCdxFile)
				ERASE (lcCdxFile)
				SELECT (lnOldSel) 
				lbRetVal = .T.
			ENDIF
		ENDIF 

		RETURN lbRetVal 
	ENDPROC
	
	*//////////////////////////////////////////////////////////////
	*// RemoveAllTags(tbDeleteCdxFile) - set order to 0, and
	*// if tbDeleteCdxFile=.T. also deletes the temporary Cdx-file 
	*// used of the given instance of class-header at grid
	*//////////////////////////////////////////////////////////////
	PROCEDURE RemoveAllTags
		LPARAMETERS tbDeleteCdxFile

		IF VARTYPE(tbDeleteCdxFile) # 'L'
			tbDeleteCdxFile = !EMPTY(tbDeleteCdxFile)
		ENDIF
		
		LOCAL lnRecNo as Number, lbIsAlias as Boolean 
		IF USED(this.cAlias)
			lbIsAlias = .T.
			lnRecNo = IIF(!EOF(this.cAlias), RECNO(this.cAlias), 0) 
			IF tbDeleteCdxFile
				this.DeleteCdxFile(this.cAlias)
			ELSE
				IF TAGCOUNT(this.cAlias) > 0
					SET ORDER TO 0 IN (this.cAlias)
				ENDIF
			ENDIF
		ENDIF

		this.ClearAllPictures()

		IF lbIsAlias ;
				AND TYPE('This.Parent.Parent') = 'O' ;
				AND !ISNULL(This.Parent.Parent)
			LOCAL oGrd as Grid, lnActiveColumn as Integer 
			lnActiveColumn = this.Parent.ColumnOrder
			oGrd = This.Parent.Parent
			WITH oGrd
			    .ActivateCell(1, lnActiveColumn)
			    IF lnRecNo # 0
			        GO (lnRecNo) IN (this.cAlias)
			    ELSE
			        GO TOP IN (this.cAlias) 
			    ENDIF
			    .Refresh()
			ENDWITH
		ENDIF	
	ENDPROC

	*//////////////////////////////////////////////////////////////
	*// Click() - event click() column header of grid,
	*// change of the order of sorting data for column
	*// Chain of changing order: ASCENDING, DESCENDING, NO SORTING
	*//////////////////////////////////////////////////////////////
	PROCEDURE Click
		*
		*-- Check call-environment
		IF TYPE('This.Parent.Parent') # 'O' OR ISNULL(This.Parent.Parent)
			RETURN .F.
		ENDIF	 
		IF TYPE('this.Parent.ControlSource') # 'C' OR EMPTY(this.Parent.ControlSource)
			RETURN .F.
		ENDIF	
		IF GETWORDCOUNT(this.Parent.ControlSource, '.') <> 2
			RETURN .F.
		ENDIF
		IF INLIST(TYPE(this.Parent.ControlSource), 'M', 'W', 'G', 'U')
			RETURN .F.
		ENDIF
		
		*
		*-- Init class-instance properties: cAlias, cField, cTag
		IF EMPTY(this.cAlias) ;
				OR EMPTY(this.cField) ;
				OR EMPTY(this.cTag)
			this.cAlias = GETWORDNUM(this.Parent.ControlSource, 1, '.')
			this.cField = GETWORDNUM(this.Parent.ControlSource, 2, '.')
			this.cTag = LOWER(this.cField)
			IF LENC(this.cTag) > 10
				this.cTag = LEFTC(this.cTag, 10)
			ENDIF
		ENDIF
			
		*
		*-- Check class-instance properties: cAlias, cField, cTag
		IF EMPTY(this.cAlias) ;
				OR EMPTY(this.cField) ;
				OR EMPTY(this.cTag)
			RETURN .F.
		ENDIF
		IF !USED(this.cAlias)
			RETURN .F.
		ENDIF

		LOCAL lcCdxFile as String;
			,lbTagFound as Boolean;
			,lnOldSel as Number;
			,lcOldTag as String;
			,lbIsDescending as Boolean
		lnOldSel = SELECT() 
		lcCdxFile = this.GetCdxFile(this.cAlias)
		lcOldTag = ''
		
		SELECT (this.cAlias)

		*
		*-- Check cdx-file
		IF FILE(lcCdxFile)  
			*
			*-- Search this.cTag in cdx-file
			LOCAL lnCntTag as Number
			lcOldTag = LOWER(SYS(22)) 
			FOR lnCntTag = 1 TO TAGCOUNT(lcCdxFile, this.cAlias)
				IF this.cTag == LOWER(TAG(lcCdxFile, lnCntTag, this.cAlias))
					IF this.cTag == lcOldTag
						lbIsDescending = DESCENDING(lcCdxFile, lnCntTag, this.cAlias)
					ELSE
						lcOldTag = ''	
					ENDIF 
					lbTagFound = .T.
					EXIT
				ENDIF
			ENDFOR
			*
			*-- Clear all pictures
			this.ClearAllPictures()
		ENDIF
		thisform.LockScreen = .T.

		*
		*--  Get current recno
		LOCAL lnRecNo as Number
		lnRecNo = IIF(!EOF(this.cAlias), RECNO(this.cAlias), 0) 
		IF !lbTagFound
			*
			*-- Create new tag
			LOCAL lcField As String
			lcField = this.cField
			INDEX ON &lcField TAG (this.cTag) ADDITIVE ASCENDING  
			this.Picture = this.cPicDn
		ELSE
			*
			*-- Change type of sorting
			DO CASE
			CASE EMPTY(lcOldTag)
				SET ORDER TO TAG (this.cTag) ASCENDING IN (this.cAlias)
				this.Picture = this.cPicDn
			CASE lbIsDescending
				SET ORDER TO 0
			OTHERWISE	
				SET ORDER TO TAG (this.cTag) DESCENDING IN (this.cAlias) 
				this.Picture = this.cPicUp
			ENDCASE
		ENDIF

		*
		*-- Refresh grid
		LOCAL oGrd as Grid;
			,lnRelativeRow as Integer;
			,lnActiveColumn as Integer  
		lnActiveColumn = this.Parent.ColumnOrder
		oGrd = This.Parent.Parent
		CLEAR TYPEAHEAD 
		WITH oGrd 
			lnRelativeRow = .RelativeRow
		    .ActivateCell(1, lnActiveColumn)
			DO WHILE (.ActiveColumn # lnActiveColumn)
				KEYBOARD '{TAB}'
				DOEVENTS  
			ENDDO
		    IF lnRecNo # 0
		        GO (lnRecNo)
		    ELSE
		        GO TOP 
		    ENDIF
		    IF lnRelativeRow - .RelativeRow = 0
				*  
     			*-- For one row visible in grid only
     			LOCAL lnCntRowUp as Integer 
		    	FOR lnCntRowUp = 1 TO N_SCROLL_UP
		    		.DoScroll(0)
		    		.Refresh()
		    	ENDFOR
		    	IF lnRecNo # 0 AND lnRecNo # RECNO()
		    		GO (lnRecNo)
		    	ENDIF
		    ENDIF
			thisform.LockScreen = .F.
		    .Refresh()
		ENDWITH
		SELECT (lnOldSel)
	ENDPROC
	
	PROCEDURE Destroy
		*
		*-- Delete cdx-file if exist
		this.DeleteCdxFile(this.cAlias)
	ENDPROC
	
ENDDEFINE 